import { FilterQuery } from 'mongodb';
import { DatabaseDriver } from './DatabaseDriver';
import { MongoConnection } from '../connections/MongoConnection';
import { EntityData, IEntityType, IPrimaryKey } from '../decorators';
import { QueryOrder } from '../query';
import { MongoPlatform } from '../platforms/MongoPlatform';
import { QueryResult } from '../connections';
export declare class MongoDriver extends DatabaseDriver<MongoConnection> {
    protected readonly connection: MongoConnection;
    protected readonly platform: MongoPlatform;
    find<T extends IEntityType<T>>(entityName: string, where: FilterQuery<T>, populate: string[], orderBy: Record<string, QueryOrder>, limit: number, offset: number): Promise<T[]>;
    findOne<T extends IEntityType<T>>(entityName: string, where: FilterQuery<T> | IPrimaryKey, populate?: string[], orderBy?: Record<string, QueryOrder>): Promise<T | null>;
    count<T extends IEntityType<T>>(entityName: string, where: FilterQuery<T>): Promise<number>;
    nativeInsert<T extends IEntityType<T>>(entityName: string, data: EntityData<T>): Promise<QueryResult>;
    nativeUpdate<T extends IEntityType<T>>(entityName: string, where: FilterQuery<T> | IPrimaryKey, data: EntityData<T>): Promise<QueryResult>;
    nativeDelete<T extends IEntityType<T>>(entityName: string, where: FilterQuery<T> | IPrimaryKey): Promise<QueryResult>;
    aggregate(entityName: string, pipeline: any[]): Promise<any[]>;
    private renameFields;
    private getCollectionName;
}
