"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts_morph_1 = require("ts-morph");
const path_1 = require("path");
const fs_1 = require("fs");
const MetadataProvider_1 = require("./MetadataProvider");
const utils_1 = require("../utils");
class TypeScriptMetadataProvider extends MetadataProvider_1.MetadataProvider {
    constructor() {
        super(...arguments);
        this.project = new ts_morph_1.Project();
    }
    async loadEntityMetadata(meta, name) {
        if (!meta.path) {
            return;
        }
        this.initProperties(meta, name);
    }
    initProperties(meta, name) {
        // init types and column names
        Object.values(meta.properties).forEach(prop => {
            if (prop.entity) {
                prop.type = utils_1.Utils.className(prop.entity());
            }
            else if (!prop.type) {
                const file = meta.path.match(/\/[^\/]+$/)[0].replace(/\.js$/, '.ts');
                prop.type = this.readTypeFromSource(file, name, prop);
            }
        });
    }
    readTypeFromSource(file, name, prop) {
        const source = this.getSourceFile(file);
        const properties = source.getClass(name).getInstanceProperties();
        const property = properties.find(v => v.getName() === prop.name);
        return property.getType().getText(property);
    }
    getSourceFile(file) {
        if (!this.sources) {
            this.initSourceFiles();
        }
        const source = this.sources.find(s => s.getFilePath().endsWith(file));
        if (!source) {
            throw new Error(`Source file for entity ${name} not found, check your 'entitiesDirsTs' option`);
        }
        return source;
    }
    initSourceFiles() {
        const tsDirs = this.config.get('entitiesDirsTs');
        if (tsDirs.length > 0) {
            const dirs = this.validateDirectories(tsDirs);
            this.sources = this.project.addExistingSourceFiles(dirs);
        }
        else {
            this.sources = this.project.addSourceFilesFromTsConfig(this.config.get('tsConfigPath'));
        }
    }
    validateDirectories(dirs) {
        return dirs.map(dir => {
            const path = path_1.join(this.config.get('baseDir'), dir);
            if (!fs_1.existsSync(path)) {
                throw new Error(`Path ${path} does not exist`);
            }
            return path_1.join(path, '**', '*.ts');
        });
    }
}
exports.TypeScriptMetadataProvider = TypeScriptMetadataProvider;
