"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const metadata_1 = require("../metadata");
const entity_1 = require("../entity");
const ChangeSet_1 = require("./ChangeSet");
class ChangeSetPersister {
    constructor(driver, identifierMap) {
        this.driver = driver;
        this.identifierMap = identifierMap;
        this.metadata = metadata_1.MetadataStorage.getMetadata();
    }
    async persistToDatabase(changeSet) {
        const meta = this.metadata[changeSet.name];
        // process references first
        for (const prop of Object.values(meta.properties)) {
            this.processReference(changeSet, prop);
        }
        // persist the entity itself
        await this.persistEntity(changeSet, meta);
    }
    async persistEntity(changeSet, meta) {
        if (changeSet.type === ChangeSet_1.ChangeSetType.DELETE) {
            await this.driver.nativeDelete(changeSet.name, changeSet.entity.__primaryKey);
        }
        else if (changeSet.type === ChangeSet_1.ChangeSetType.UPDATE) {
            const res = await this.driver.nativeUpdate(changeSet.name, changeSet.entity.__primaryKey, changeSet.payload);
            this.mapReturnedValues(changeSet.entity, res, meta);
        }
        else if (changeSet.entity.__primaryKey) { // ChangeSetType.CREATE with primary key
            const res = await this.driver.nativeInsert(changeSet.name, changeSet.payload);
            this.mapReturnedValues(changeSet.entity, res, meta);
            delete changeSet.entity.__initialized;
        }
        else { // ChangeSetType.CREATE without primary key
            const res = await this.driver.nativeInsert(changeSet.name, changeSet.payload);
            this.mapReturnedValues(changeSet.entity, res, meta);
            changeSet.entity.__primaryKey = res.insertId;
            this.identifierMap[changeSet.entity.__uuid].setValue(changeSet.entity.__primaryKey);
            delete changeSet.entity.__initialized;
        }
    }
    processReference(changeSet, prop) {
        const value = changeSet.payload[prop.name];
        if (value instanceof entity_1.EntityIdentifier) {
            changeSet.payload[prop.name] = value.getValue();
        }
        else if (Array.isArray(value) && value.some(item => item instanceof entity_1.EntityIdentifier)) {
            changeSet.payload[prop.name] = value.map(item => item instanceof entity_1.EntityIdentifier ? item.getValue() : item);
        }
        if (prop.onUpdate) {
            changeSet.entity[prop.name] = changeSet.payload[prop.name] = prop.onUpdate();
        }
    }
    mapReturnedValues(entity, res, meta) {
        if (res.row && Object.keys(res.row).length > 0) {
            Object.values(meta.properties).forEach(prop => {
                if (res.row[prop.fieldName]) {
                    entity[prop.name] = res.row[prop.fieldName];
                }
            });
        }
    }
}
exports.ChangeSetPersister = ChangeSetPersister;
